/**
 * @module libflitter/Context
 */

/**
 * Receptacle for methods and properties that are bound to a particular instance of {@link module:libflitter/app/FlitterApp~FlitterApp}.
 * Used to export the functionality of a Unit to the rest of Flitter.
 * @class
 * @deprecated
 */
class Context {

    /**
     * Initialize the context. 
     * Store the {@link module:libflitter/app/FlitterApp~FlitterApp} to which items should be bound.
     * @param {module:libflitter/app/FlitterApp~FlitterApp} app - the Flitter app
     */
    constructor(app){
        console.warn('Use of application contexts is deprecated and will be removed in a future release.')
        console.warn('Please prefer use of injectable services.')

        /**
         * Collection of methods and properties stored in the context.
         * @type {Object}
         * @name Context#bindings
         */
        this.bindings = {}
    }

    /**
     * Store a new method or property in the context.
     * If storing a method, bind the method to {@link module:libflitter/Context~Context#app}.
     * @param {string} name - the name that should be used to access this resource
     * @param {*} handler - method or property to be attached
     */
    bind(name, handler){
        
        if ( typeof handler === 'function' ){
            this.bindings[name] = handler.bind(this.app)
        }
        else {
            this.bindings[name] = handler
        }
        
    }

    /**
     * Get a Proxy containing the bound methods and properties. 
     * This is an unchangeable item which is used to access resources stored in this context.
     * @returns {Proxy}
     */
    serialize(){
        let handler = {
            get( target, key, receiver ){
                const method = target[key]
                if ( typeof method === 'function' ){
                    return (...args) => {
                        return method.apply(this, args)
                    }
                }
                else {
                    return method
                }
            }
        }

        return new Proxy(this.bindings, handler)
    }

}

module.exports = exports = Context
